/* A program to generate a grouping map by identifying conditions
 * Save the map in a pgm file for use with optimization algorithm
 * 2015-05-04: Created by Abdalla Ahmed
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#include <time.h>

int t, r, q, p, ONE;                                                            // Let these be global to be accessible for all functions
unsigned char *seq;

int findThresholds(int range) {
    seq = new unsigned char[t]();
    int n = 1;
    int dX;
    for (int x = 1; x <= range; x++) {
        dX = (p * x) % ONE;
        if (dX < t) { seq[t - dX] = 1; }
        if (dX > ONE - t) { seq[ONE - dX] = 1; }
        dX = (r * x) % ONE;
        if (dX < t) { seq[t - dX] = 1; }
        if (dX > ONE - t) { seq[ONE - dX] = 1; }
    }
    unsigned char clr = 0;
    for (int i = 1; i < t; i++) {
        if (seq[i] == 1) { clr ^= 255; n++; }
        seq[i] = clr;
    }
    return n;
}


int plotMap(const char *fname) {
    FILE *pgmfile = fopen(fname, "w");
    fprintf(pgmfile, "P5\n%d %d\n255\n", t, t);
    for (int dX = 0; dX < t; dX++) {
        fprintf(pgmfile, "%c", seq[dX]);
    }
    for (int dY = 1; dY < t; dY++) {
        unsigned char invert =  (seq[dY] == seq[0]) ? 0 : 255;
        for (int dX = 0; dX < t; dX++) {
            fprintf(pgmfile, "%c", (seq[dX] ^ invert));
        }
    }
    fclose(pgmfile);
}

const char *USAGE_MESSAGE = "Usage: %s [-p [path/]fileNamePrefix] <t> <r> <half-range>\n";

int main(int argc,char **argv) {
    int opt;
    char *fileNamePrefix = (char *)"";
    while ((opt = getopt(argc, argv, "p:")) != -1) {
        switch (opt) {
            case 'p': fileNamePrefix = optarg; break;
            default: fprintf(stderr, USAGE_MESSAGE, argv[0]); exit(1);
        }
    }
    if (optind > argc - 3) {
        fprintf(stderr, USAGE_MESSAGE, argv[0]); exit(1);
    }
    t = atoi(argv[optind]);
    r = atoi(argv[optind+1]);
    int range = atoi(argv[optind+2]);
    q = t + r;
    p = q + t;
    ONE = 2*q;
    int n = findThresholds(range);
    char fileName[200];
    sprintf(fileName, "%smap-%d-%d.pgm", fileNamePrefix, t, r);
    plotMap(fileName);
    fprintf(stderr, "%dx%d neighborhood generated %d x %d = %d IDs\n",
            2*range+1, 2*range+1, n, n, n*n);
    return 0;
}
