/* A program to produce thresholds for a given neighborhood size
 * Implements algorithm 1 in the paper for use with generation algorithm
 * Abdalla Ahmed
 * 2015-06-02
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#include <algorithm>

int t, r, q, p, ONE, range;                                                     // Let these be global to be accessible for all functions
std::vector<int> th;

void findThresholds() {
    th.reserve(2 * range);                                                      // Thresholds are no more than the scanned width
    th.push_back(0);
    int dX;
    for (int x = 1; x <= range; x++) {
        dX = (p * x) % ONE;
        if (dX < t) { th.push_back(t - dX); }
        if (dX > ONE - t) { th.push_back(ONE - dX); }
        dX = (r * x) % ONE;
        if (dX < t) { th.push_back(t - dX); }
        if (dX > ONE - t) { th.push_back(ONE - dX); }
    }
    th.push_back(t);
    std::sort(th.begin(), th.end());
}

const char *USAGE_MESSAGE = "Usage: %s <t> <r> <half-range>\n";

int main(int argc,char **argv) {
    int opt;
    while ((opt = getopt(argc, argv, "")) != -1) {
        switch (opt) {
            default: fprintf(stderr, USAGE_MESSAGE, argv[0]); exit(1);
        }
    }
    if (optind > argc - 3) {
        fprintf(stderr, USAGE_MESSAGE, argv[0]); exit(1);
    }
    t = atoi(argv[optind]);
    r = atoi(argv[optind+1]);
    range = atoi(argv[optind+2]);
    q = t + r;
    p = q + t;
    ONE = 2*q;
    findThresholds();
    printf(
        "a %d neighborhood window generates %d IDs.\nThresholds:\n",
        2*range+1, (int)th.size() - 1
    );
    for (int i = 0; i < th.size(); i++) {
        printf("%d\n", th[i]);
    }
    return 0;
}
